@@ -15,37 +15,26 @@
 #'\dontrun{
 #'##- required packages
 #'require(uavRst)
-#'require(link2GI)
-#'
-#'##- project folder
-#'projRootDir<-tempdir()
-#'
-#'##- create subfolders please mind that the pathes are exported as global variables
-#'paths<-link2GI::initProj(projRootDir = projRootDir,
-#'                         projFolders = c("data/","data/ref/","output/","run/","las/"),
-#'                         global = TRUE,
-#'                         path_prefix = "path_")
-#'##- overide trailing backslash issue
-#'path_run<-ifelse(Sys.info()["sysname"]=="Windows", sub("/$", "",path_run),path_run)
-#'setwd(path_run)
-#'unlink(paste0(path_run,"*"), force = TRUE)
+
+#'setwd(tempdir())
 #'
 #'##- get the tutorial data
 #'utils::download.file("https://github.com/gisma/gismaData/raw/master/uavRst/data/tutorial_data.zip",
-#'                      paste0(path_run,"tutorial_data.zip"))
-#'unzip(paste0(path_run,"tutorial_data.zip"),exdir =  path_run)
+#'                      "tutorial_data.zip")
+#'unzip("tutorial_data.zip",exdir =  ".")
 #'
 #'##- get the files
 #'imageTrainStack <- list()
 #'geomTrainStack <- list()
-#'imageTrainFiles <- Sys.glob(paste0(path_run,"rgb*.tif"))
-#'geomTrainFiles <- Sys.glob(paste0(path_run,"rgb*.shp"))
+#'imageTrainFiles <- Sys.glob("rgb??.tif")
+#'geomTrainFiles <- Sys.glob("rgb??.shp")
 #'
 #'##- create stacks from image and geometry files
 #'imageTrainStack<-lapply(imageTrainFiles, FUN=raster::stack)
 #'geomTrainStack  <- lapply(geomTrainFiles, FUN=raster::shapefile)
 #'names(imageTrainStack[[1]])<-c("red","green","blue")
 #'names(imageTrainStack[[2]])<-c("red","green","blue")
+#'names(imageTrainStack[[3]])<-c("red","green","blue")
 #'
 #'##' finally extraxt the training data to a data frame
 #'trainDF <- get_traindata(rasterStack  = imageTrainStack,
@@ -104,30 +93,22 @@ get_traindata<-function(rasterStack  = NULL,
 #'\dontrun{
 #' ##- required packages
 #'  require(uavRst)
-#'  require(link2GI)
+
 #'
 #' ##- project root folder
-#'  projRootDir<-tempdir()
+#'  setwd(tempdir())
 #'
-#' ##- create subfolders please mind that the pathes are exported as global variables
-#'  paths<-link2GI::initProj(projRootDir = projRootDir,
-#'                           projFolders = c("data/","data/ref/","output/","run/","las/"),
-#'                           global = TRUE,
-#'                           path_prefix = "path_")
-#' ##- overide trailing backslash issue
-#'  path_run<-ifelse(Sys.info()["sysname"]=="Windows", sub("/$", "",path_run),path_run)
-#'  setwd(path_run)
-#'  unlink(paste0(path_run,"*"), force = TRUE)
 #'
 #' ##- get the rgb image, chm and training data
 #'  utils::download.file("https://github.com/gisma/gismaData/raw/master/uavRst/data/tutorial.zip",
-#'                       paste0(path_run,"tutorial_data.zip"))
-#'  unzip(zipfile = paste0(path_run,"tutorial_data.zip"),exdir = path_run)
+#'                       "tutorial_data.zip")
+#'  unzip(zipfile = "tutorial_data.zip" ,
+#'          exdir = ".")
 
 #'
 #' # read data
-#'  position <- raster::shapefile(paste0(path_run,"position.shp"))
-#'  imageFiles <-Sys.glob(paths = paste0(path_run,"rgb*","tif"))
+#'  position <- raster::shapefile("position.shp")
+#'  imageFiles <-Sys.glob(paths = paste0("rgb*","tif"))
 #'  imageTrainStack<-lapply(imageFiles, FUN=raster::stack)
 #'
 #' ## get counts
@@ -136,7 +117,8 @@ get_traindata<-function(rasterStack  = NULL,
 #'                imageFiles = imageFiles,
 #'                 outPrefix = "",
 #'                       ext = ".tif",
-#'                      path = path_run)
+#'                      path = tempdir())
+#' head(df1)
 #'##+}
 
 
@@ -146,13 +128,13 @@ get_counts<- function(ids=c(1,2),
                       buffersize=1.5,
                       outPrefix="classified_index_",
                       ext=".tif",
-                      path = path_output,
+                      path = tempdir(),
                       dropChars=0) {
 
   buffers<-rgeos::gBuffer(position,width=buffersize)
   ex<-data.frame()
   df<- lapply(seq(1:length(position)), function(i) {
-    fn<-paste0(path,outPrefix,substr(position[i,]$tree,1,nchar(position[i,]$tree)-dropChars),ext)
+    fn<- file.path(R.utils::getAbsolutePath(path),paste0(outPrefix,substr(position[i,]$tree,1,nchar(position[i,]$tree)-dropChars),ext))
 
     if (file.exists(fn)){
       pos <-sp::spTransform(position[i,],raster::raster(fn)@crs)
@@ -195,15 +177,14 @@ get_counts<- function(ids=c(1,2),
 #'                         projFolders = c("data/","data/ref/","output/","run/","las/"),
 #'                         global = TRUE,
 #'                         path_prefix = "path_")
-#' ##- overide trailing backslash issue
-#' path_run<-ifelse(Sys.info()["sysname"]=="Windows", sub("/$", "",path_run),path_run)
+
 #' setwd(path_run)
 #' unlink(paste0(path_run,"*"), force = TRUE)
 #'
 #' ##- get the tutorial data
 #' utils::download.file("https://github.com/gisma/gismaData/raw/master/uavRst/data/ffs.zip",
 #' paste0(path_run,"ffs.zip"))
-#' unzip(zipfile =  paste0(path_run,"ffs.zip"), exdir = path_run)
+#' unzip(zipfile =  paste0(path_run,"ffs.zip"), exdir = ".")
 #'
 #' ##- assign tutorial data
 #' imageFile <- paste0(path_run,"predict.tif")
@@ -298,15 +279,14 @@ predict_rgb <- function(imageFiles=NULL,
 #'                          projFolders = c("data/","data/ref/","output/","run/","las/"),
 #'                          global = TRUE,
 #'                          path_prefix = "path_")
-#' ##- overide trailing backslash issue
-#' path_run<-ifelse(Sys.info()["sysname"]=="Windows", sub("/$", "",path_run),path_run)
+
 #' setwd(path_run)
 #' unlink(paste0(path_run,"*"), force = TRUE)
 #'
 #' ##- get the rgb image, chm and training data
 #' utils::download.file("https://github.com/gisma/gismaData/raw/master/uavRst/data/ffs.zip",
 #'                       paste0(path_run,"ffs.zip"))
-#' unzip(zipfile = paste0(path_run,"ffs.zip"),exdir = path_run)
+#' unzip(zipfile = paste0(path_run,"ffs.zip"),exdir = ".")
 #'
 #' ##- get geometrical training data assuming that you have used before the calc_ext function
 #' trainDF<-readRDS(paste0(path_run,"tutorial_trainDF.rds"))
@@ -424,7 +404,10 @@ ffs_train<-function(   trainingDF   = NULL,
 #' Convenient function to preprocess synthetic raster bands from a given RGB image and/or DTM/DSM data.
 #' @description
 #' Convenient function to preprocess synthetic raster bands from a given RGB image and/or DTM/DSM data.
-#' Optionally the raster values  can be extracted to data frames on base of vector data. This is useful
+#' Currently for all chanels of the input images the following textures and indices can be calculated:\cr
+#' rgb indices (17), color transformation (9), haralick (3 = 25 layers), statitics (4),edge (3),morpho (4), DEM Parameter (20).
+#' All layers will be stacked (as an ENVI file). 
+#' If wanted the raster values can be extracted to a data frames by overlaying corresponding vector data. NOTE: The vector data has to be named identically to the rasterfiles.  This is useful
 #' for for classification training purposes and covers usually step 1 of the random forest based
 #' classification of UAV derived visible imagery and point clouds.
 
@@ -487,30 +470,21 @@ ffs_train<-function(   trainingDF   = NULL,
 #' # create and check the links to the GI software
 #' giLinks<-uavRst::linkAll()
 #' if (giLinks$saga$exist & giLinks$otb$exist){
-#'
-#' ##- set root folder
-#' projRootDir<-tempdir()
-#'
-#' ##- create and set subfolders
+#'#'
+#' ##- create and set folders
 #' ##- please mind that the pathes are exported as global variables
-#' paths<-link2GI::initProj(projRootDir = projRootDir,
+#' paths<-link2GI::initProj(projRootDir = tempdir(),
 #'                          projFolders = c("data/","data/ref/","output/","run/","las/"),
 #'                          global = TRUE,
 #'                          path_prefix = "path_")
 #'
-#' ##- overide trailing backslash issue
-#' path_run<-ifelse(Sys.info()["sysname"]=="Windows", sub("/$", "",path_run),path_run)
-#' setwd(path_run)
-#'
 #' ##- clean runtime folder
 #' unlink(paste0(path_run,"*"), force = TRUE)
 #'
-#' # get the tutorial data
+#' ##- get the tutorial data
 #' utils::download.file("https://github.com/gisma/gismaData/raw/master/uavRst/data/tutorial_data.zip",
 #'                      paste0(path_run,"tutorial_data.zip"))
-#' unzip(zipfile = paste0(path_run,"tutorial_data.zip"), exdir = path_run)
-#'
-
+#' unzip(zipfile = paste0(path_run,"tutorial_data.zip"), exdir = R.utils::getAbsolutePath(path_run))
 #'
 #' ##- calculate some synthetic channels from the RGB image and the canopy height model
 #' ##- then extract the from the corresponding training geometries the data values aka trainingdata
@@ -585,7 +559,7 @@ calc_ext<- function ( calculateBands    = FALSE,
                       currentIdxFolder  = NULL,
                       cleanRunDir        = TRUE,
                       giLinks = NULL){
-
+  if (!exists("path_run")) path_run = tempdir()
   if (!rgbi) rgbTrans <- hara <- stat <- edge <- morpho <- FALSE
   if (is.null(giLinks)){
     giLinks <- linkAll()
@@ -614,7 +588,7 @@ calc_ext<- function ( calculateBands    = FALSE,
   currentDataFolder<- currentDataFolder #paste0(path_data_training)
   currentIdxFolder<- currentIdxFolder # paste0(path_data_training_idx)
 
-  if (((stat == TRUE) || (hara == TRUE) || (edge == TRUE) || (morpho == TRUE)) & path_OTB == "") stop("OTB missing - please check")
+  if (((stat == TRUE) || (hara == TRUE) || (edge == TRUE) || (morpho == TRUE)) & !otb$exist == "") stop("OTB missing - please check")
 
   ### ----- start preprocessing ---------------------------------------------------
   if (calculateBands) {
@@ -641,7 +615,7 @@ calc_ext<- function ( calculateBands    = FALSE,
 
         #cat(catNote(":::: processing dem... ",demType,"\n"))
         flist<-append(flist, Sys.glob(demFiles[i]))
-        dellist <- append(dellist, paste0(path_run,"dem2.tif"))
+        dellist <- append(dellist, file.path(R.utils::getAbsolutePath(path_run),"dem2.tif"))
         bandNames <-append(bandNames,"dem")
         morpho_dem(dem = demFiles[i],
                    item = demType,
@@ -650,8 +624,8 @@ calc_ext<- function ( calculateBands    = FALSE,
                    maxScale = maxScale,
                    numScale = numScale,
                    giLinks = giLinks)
-        flist<-append(flist, Sys.glob(paste0(path_run,demType,".tif")))
-        dellist <- append(dellist, Sys.glob(paste0(path_run,demType,".*")))
+        flist<-append(flist, Sys.glob(file.path(R.utils::getAbsolutePath(path_run),paste0(demType,".tif"))))
+        dellist <- append(dellist, Sys.glob(file.path(R.utils::getAbsolutePath(path_run),paste0(demType,".*"))))
         for (item in demType)
           bandNames <-append(bandNames,make_bandnames(dem = item))
 
@@ -671,12 +645,12 @@ calc_ext<- function ( calculateBands    = FALSE,
         names(rgb_rgbi)<-append(c("red","green","blue"),indices)
         cat(catOk("\n     save ...",paste0(path_run,"rgbi_",basename(imageFiles[i])),"\n"))
         raster::writeRaster(rgb_rgbi,
-                            paste0(path_run,"rgbi_",basename(imageFiles[i])),
+                            file.path(R.utils::getAbsolutePath(path_run),paste0("rgbi_",basename(imageFiles[i]))),
                             progress = "text",
                             overwrite=TRUE)
 
-        flist<-append(flist, paste0(path_run,"rgbi_",basename(imageFiles[i])))
-        dellist <- append(dellist, paste0(path_run,"rgbi_",basename(imageFiles[i])))
+        flist<-append(flist, file.path(R.utils::getAbsolutePath(path_run),paste0("rgbi_",basename(imageFiles[i]))))
+        dellist <- append(dellist, file.path(R.utils::getAbsolutePath(path_run),paste0("rgbi_",basename(imageFiles[i]))))
       }
       # if RGB transform
       if (rgbTrans){
@@ -687,7 +661,7 @@ calc_ext<- function ( calculateBands    = FALSE,
         rgbTranslist<-list()
         jj=1
         for (colMod in colorSpaces) {
-          rgbTranslist[[jj]]<-paste0(path_run,colMod,"_",basename(imageFiles[i]))
+          rgbTranslist[[jj]]<-file.path(R.utils::getAbsolutePath(path_run),paste0(colMod,"_",basename(imageFiles[i])))
           jj<-jj+1
         }
         rt<- lapply(rgbTranslist, FUN=raster::stack)
@@ -696,14 +670,14 @@ calc_ext<- function ( calculateBands    = FALSE,
           raster::projection(rt[[jj]]) <- raster::crs(raster::projection(r))
           cat(catOk(":::: save... ",paste0(colorSpaces[jj],"_",basename(imageFiles[i])),"\n"))
           raster::writeRaster(raster::stack(rt[[jj]][[1:(raster::nlayers(rt[[jj]])-1)]]),
-                              paste0(path_run,colorSpaces[jj],"_ref",basename(imageFiles[i])),
+                              file.path(R.utils::getAbsolutePath(path_run),paste0(colorSpaces[jj],"_ref",basename(imageFiles[i]))),
                               overwrite=TRUE,
                               options="INTERLEAVE=BAND",
                               progress="text")
           bandNames <-append(bandNames,make_bandnames(rgbTrans = colorSpaces[jj]))
-          flist<-append(flist, paste0(path_run,colorSpaces[jj],"_ref",basename(imageFiles[i])))
-          dellist <- append(dellist, paste0(path_run,colorSpaces[jj],"_ref",basename(imageFiles[i])))
-          dellist <- append(dellist, paste0(path_run,colorSpaces[jj],basename(imageFiles[i])))
+          flist<-append(flist, file.path(R.utils::getAbsolutePath(path_run),paste0(colorSpaces[jj],"_ref",basename(imageFiles[i]))))
+          dellist <- append(dellist, file.path(R.utils::getAbsolutePath(path_run),paste0(colorSpaces[jj],"_ref",basename(imageFiles[i]))))
+          dellist <- append(dellist, file.path(R.utils::getAbsolutePath(path_run),paste0(colorSpaces[jj],basename(imageFiles[i]))))
         }
         #file.remove(paste0(path_run,unlist(rgbTranslist)))
         #r<-raster::stack(imageFiles[i])
@@ -722,20 +696,20 @@ calc_ext<- function ( calculateBands    = FALSE,
           # if (filterBand!="") {
           cat(catNote(":::: write temporary channel...",paste0(filterBand,"_",basename(imageFiles[i])),"\n"))
           raster::writeRaster(rgb_rgbi[[bandNr]],
-                              paste0(path_run,filterBand,"_",basename(imageFiles[i])),
+                              file.path(R.utils::getAbsolutePath(path_run),paste0(filterBand,"_",basename(imageFiles[i]))),
                               progress = "text",
                               overwrite=TRUE)
-          fbFN<-paste0(path_run,filterBand,"_",basename(imageFiles[i]))
+          fbFN<-file.path(R.utils::getAbsolutePath(path_run),paste0(filterBand,"_",basename(imageFiles[i])))
          # filterband
         if (stat){
           cat(catNote(":::: processing stats...",fbFN,"\n"))
           otb_stat(input = fbFN,
-                   out = paste0(path_run,filterBand,"stat_",basename(imageFiles[i])),
+                   out = file.path(R.utils::getAbsolutePath(path_run),paste0(filterBand,"stat_",basename(imageFiles[i]))),
                    ram = "4096",
                    radius =  kernel,
                    giLinks=giLinks)
-          flist<-append(flist,Sys.glob(paste0(path_run,filterBand,"stat_*")))
-          dellist <-append(dellist,Sys.glob(paste0(path_run,filterBand,"stat_*")))
+          flist<-append(flist,Sys.glob(file.path(R.utils::getAbsolutePath(path_run),paste0(filterBand,"stat_*"))))
+          dellist <-append(dellist,Sys.glob(file.path(R.utils::getAbsolutePath(path_run),paste0(filterBand,"stat_*"))))
           bandNames <-append(bandNames,paste0(make_bandnames(stat = TRUE),"_",filterBand))
         }
         # if calc edge
@@ -743,12 +717,12 @@ calc_ext<- function ( calculateBands    = FALSE,
           for (edges in edgeType){
             cat(catNote(":::: processing edge... ",edges,"\n"))
             uavRst::otbtex_edge(input = fbFN,
-                                out = paste0(path_run,filterBand,edges,basename(imageFiles[i])),
+                                out = file.path(R.utils::getAbsolutePath(path_run),paste0(filterBand,edges,basename(imageFiles[i]))),
                                 filter = edges,
                                 giLinks=giLinks)
 
-            flist<-append(flist,Sys.glob(paste0(path_run,filterBand,edges,"*")))
-            dellist<-append(dellist,Sys.glob(paste0(path_run,filterBand,edges,"*")))
+            flist<-append(flist,Sys.glob(file.path(R.utils::getAbsolutePath(path_run),paste0(filterBand,edges,"*"))))
+            dellist<-append(dellist,Sys.glob(file.path(R.utils::getAbsolutePath(path_run),paste0(filterBand,edges,"*"))))
             bandNames <-append(bandNames,make_bandnames(edge = paste0(edges,"_",filterBand)))
           }
         }
@@ -758,11 +732,11 @@ calc_ext<- function ( calculateBands    = FALSE,
           for (morphos in morphoType){
             cat(catNote(":::: processing morpho... ",morphos,"\n"))
             uavRst::otbtex_gray(input = fbFN,
-                                out = paste0(path_run,filterBand,morphos,basename(imageFiles[i])),
+                                out = file.path(R.utils::getAbsolutePath(path_run),paste0(filterBand,morphos,basename(imageFiles[i]))),
                                 filter = morphos,
                                 giLinks=giLinks)
-            flist<-append(flist,Sys.glob(paste0(path_run,filterBand,morphos,"*")))
-            dellist<-append(dellist,Sys.glob(paste0(path_run,filterBand,morphos,"*")))
+            flist<-append(flist,Sys.glob(file.path(R.utils::getAbsolutePath(path_run),paste0(filterBand,morphos,"*"))))
+            dellist<-append(dellist,Sys.glob(file.path(R.utils::getAbsolutePath(path_run),paste0(filterBand,morphos,"*"))))
             bandNames <-append(bandNames,make_bandnames(edge = paste0(morphos,"_",filterBand)))
           }
         }
@@ -771,16 +745,16 @@ calc_ext<- function ( calculateBands    = FALSE,
           for (type in haraType){
             cat(catNote(":::: processing haralick... ",type,"\n"))
             otbtex_hara(x = fbFN,
-                        output_name=paste0(path_run,filterBand,"hara_",basename(imageFiles[i])),
+                        output_name=file.path(R.utils::getAbsolutePath(path_run),paste0(filterBand,"hara_",basename(imageFiles[i]))),
                         texture = type,
                         giLinks=giLinks)
-            flist<-append(flist,Sys.glob(paste0(path_run,filterBand,"hara_*")))
-            dellist<-append(dellist,Sys.glob(paste0(path_run,filterBand,"hara_*")))
+            flist<-append(flist,Sys.glob(file.path(R.utils::getAbsolutePath(path_run),paste0(filterBand,"hara_*"))))
+            dellist<-append(dellist,Sys.glob(file.path(R.utils::getAbsolutePath(path_run),paste0(filterBand,"hara_*"))))
             bandNames <-append(bandNames,paste0(make_bandnames(bandNames = type),"_",filterBand))
           }
         }
           # delete single channel for synthetic channel calculation
-          file.remove(paste0(path_run,filterBand,"_",basename(imageFiles[i])))
+          file.remove(file.path(R.utils::getAbsolutePath(path_run),paste0(filterBand,"_",basename(imageFiles[i]))))
         }
       }# end of single channnel calculation
 
